@@ -20,6 +20,18 @@ from zope.component import getMultiAdapter
 from Acquisition import aq_inner
 from ZTUtils import make_query
 
+from z3c.form import form, field, button, group
+from plone.app.z3cform.layout import wrap_form
+from plonetheme.arnolfini.interfaces import IWeekDayContent
+from plone.app.z3cform.wysiwyg.widget import WysiwygFieldWidget
+from z3c.form.interfaces import INPUT_MODE
+from zope.component import getUtility
+from plone.registry.interfaces import IRegistry
+from Products.CMFCore.interfaces import ISiteRoot
+from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
+from plone.app.registry.browser.controlpanel import RegistryEditForm
+from Products.statusmessages.interfaces import IStatusMessage
+
 try:
     from Products.PloneGetPaid.interfaces import IBuyableMarker
     GETPAID_EXISTS = True
@@ -70,6 +82,13 @@ class CommonBrowserView(BrowserView):
         
         return self.prv
     
+    def showManageButton(self):
+        secman = getSecurityManager()
+        if not secman.checkPermission('Portlets: Manage portlets', self.context):
+            return False
+        else:
+            return True
+    
     def slideshow(self, parent):
         """
         Creates a slideshow with the media from parent
@@ -195,7 +214,12 @@ class CommonBrowserView(BrowserView):
         else:
             samedate = event.start().strftime('%d - %m - %Y') == event.end().strftime('%d - %m - %Y')
             
-        finalDatesFmt = '<div class="dates %(class)s"><span class="dateText">%(dates)s%(hours)s</span></div>'
+        exceptions = ""
+        
+        if hasattr(event, 'exceptions'):
+            exceptions = event.exceptions
+            
+        finalDatesFmt = '<div class="dates %(class)s"><span class="dateText">%(dates)s%(hours)s %(exceptions)s</span></div>'
         
         dates = "%s"%(samedate and (event.start() is not None and event.start().strftime('%A, %d %B %Y') or event.end().strftime('%A, %d %B %Y')) or "%s to %s"%(event.start().strftime('%A, %d %B %Y'), event.end().strftime('%A, %d %B %Y')))
         
@@ -205,7 +229,7 @@ class CommonBrowserView(BrowserView):
         
         hours = hoursExist and ', <span class="hours">%s to %s</span>'%(openingHour, closingHour) or ''
         
-        finalDates = finalDatesFmt%{'class': self.isEventPast(event) and 'past' or 'future', 'dates': dates, 'hours': hours}
+        finalDates = finalDatesFmt%{'class': self.isEventPast(event) and 'past' or 'future', 'dates': dates, 'hours': hours, 'exceptions':exceptions}
         
         return finalDates
             
@@ -417,8 +441,69 @@ class EventListingView(FolderListing):
         #final_res = sorted(res_list, key=lambda x: getattr(x, "start", False))
         
         return res_list
+    
+    def getNoEventMessage(self):
+        registry = getUtility(IRegistry)
+        return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.no_event'])
         
+    
+    def transformed(self, text, mt='text/x-html-safe'):
+        """Use the safe_html transform to protect text output. This also
+            ensures that resolve UID links are transformed into real links.
+        """
+        if text is None:
+            return u''
+            
+        orig = text
+        context = aq_inner(self.context)
+        if not isinstance(orig, unicode):
+            # Apply a potentially lossy transformation, and hope we stored
+            # utf-8 text. There were bugs in earlier versions of this portlet
+            # which stored text directly as sent by the browser, which could
+            # be any encoding in the world.
+            orig = unicode(orig, 'utf-8', 'ignore')
+            logger.warn("Static portlet at %s has stored non-unicode text. "
+                "Assuming utf-8 encoding." % context.absolute_url())
+
+        # Portal transforms needs encoded strings
+        orig = orig.encode('utf-8')
+
+        transformer = getToolByName(context, 'portal_transforms')
+        data = transformer.convertTo(mt, orig,
+                                     context=context, mimetype='text/html')
+        result = data.getData()
+        if result:
+            if isinstance(result, str):
+                return unicode(result, 'utf-8')
+            return result
+        return None
+    
+    def substituteWithMessage(self):
+        registry = getUtility(IRegistry)
+        weekday = DateTime().strftime("%u")
+        if weekday == '1' and registry['plonetheme.arnolfini.interfaces.IWeekDayContent.mondayActive']:
+            return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.monday'])
         
+        elif weekday == '2' and registry['plonetheme.arnolfini.interfaces.IWeekDayContent.tuesdayActive']:
+            return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.tuesday'])
+        
+        elif weekday == '3' and registry['plonetheme.arnolfini.interfaces.IWeekDayContent.wednesdayActive']:
+            return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.wednesday'])
+        
+        elif weekday == '4' and registry['plonetheme.arnolfini.interfaces.IWeekDayContent.thursdayActive']:
+            return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.thursday'])
+        
+        elif weekday == '5' and registry['plonetheme.arnolfini.interfaces.IWeekDayContent.fridayActive']:
+            return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.friday'])
+        
+        elif weekday == '6' and registry['plonetheme.arnolfini.interfaces.IWeekDayContent.saturdayActive']:
+            return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.saturday'])
+        
+        elif weekday == '7' and registry['plonetheme.arnolfini.interfaces.IWeekDayContent.sundayActive']:
+            return self.transformed(registry['plonetheme.arnolfini.interfaces.IWeekDayContent.sunday'])
+        
+        else:
+            return u''
     
 class SearchView(CommonBrowserView, Search):
     """
@@ -533,9 +618,104 @@ class FrontPageView(FolderListing):
         else:
             return None
         
-    def showManageButton(self):
-        secman = getSecurityManager()
-        if not secman.checkPermission('Portlets: Manage portlets', self.context):
-            return False
-        else:
-            return True
\ No newline at end of file
+        
+class MondayGroup(group.Group):
+    label = u'Monday'
+    fields = field.Fields(IWeekDayContent).select(
+        'mondayActive', 'monday')
+    
+class TuesdayGroup(group.Group):
+    label = u'Tuesday'
+    fields = field.Fields(IWeekDayContent).select(
+        'tuesdayActive', 'tuesday')
+    
+class WednesdayGroup(group.Group):
+    label = u'Wednesday'
+    fields = field.Fields(IWeekDayContent).select(
+        'wednesdayActive', 'wednesday')
+    
+class ThursdayGroup(group.Group):
+    label = u'Thursday'
+    fields = field.Fields(IWeekDayContent).select(
+        'thursdayActive', 'thursday')
+    
+class FridayGroup(group.Group):
+    label = u'Friday'
+    fields = field.Fields(IWeekDayContent).select(
+        'fridayActive', 'friday')
+    
+class SaturdayGroup(group.Group):
+    label = u'Saturday'
+    fields = field.Fields(IWeekDayContent).select(
+        'saturdayActive', 'saturday')
+    
+class SundayGroup(group.Group):
+    label = u'Sunday'
+    fields = field.Fields(IWeekDayContent).select(
+        'sundayActive', 'sunday')
+
+class NoEventGroup(group.Group):
+    label = u'No events'
+    fields = field.Fields(IWeekDayContent).select('no_event')
+
+class WeekDayForm(RegistryEditForm):
+    """
+    This is a view with a form to fill a body text that depends on the weekday.
+    """
+    schema = IWeekDayContent
+    groups = (
+        MondayGroup,
+        TuesdayGroup,
+        WednesdayGroup,
+        ThursdayGroup,
+        FridayGroup,
+        SaturdayGroup,
+        SundayGroup,
+        NoEventGroup)
+    label = u"Weekday specific messages"
+    description = u"Enter messages to replace today's events on each weekday. You can also add a message for days in which there are no events."
+    
+    
+    def updateFields(self):
+        super(WeekDayForm, self).updateFields()
+        self.groups[0].fields['monday'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+        self.groups[1].fields['tuesday'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+        self.groups[2].fields['wednesday'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+        self.groups[3].fields['thursday'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+        self.groups[4].fields['friday'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+        self.groups[5].fields['saturday'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+        self.groups[6].fields['sunday'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+        self.groups[7].fields['no_event'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
+    
+    @button.buttonAndHandler(u'save')
+    def handleSave(self, action):
+        data, errors = self.extractData()
+        if errors:
+            self.status = self.formErrorsMessage
+            return
+        changes = self.applyChanges(data)
+        IStatusMessage(self.request).addStatusMessage(u"Changes saved.", "info")
+        
+    @button.buttonAndHandler(u'cancel')
+    def handleCancel(self, action):
+        IStatusMessage(self.request).addStatusMessage(u"Edit cancelled.", "info")
+        self.request.response.redirect("%s" % self.context.absolute_url())
+    
+    """ 
+    @button.buttonAndHandler(u'Save')
+    def handleSave(self, action):
+        data, errors = self.extractData()
+        registry = getUtility(IRegistry)
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.monday'] = data['monday']
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.tuesday'] = data['tuesday']
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.wednesday'] = data['wednesday']
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.thursday'] = data['thursday']
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.friday'] = data['friday']
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.saturday'] = data['saturday']
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.sunday'] = data['sunday']
+        registry['plonetheme.arnolfini.interfaces.IWeekDayContent.no_event'] = data['no_event']
+        self.status = u"Your settings have been saved successfully."
+    """
+
+WeekDayFormView = wrap_form(WeekDayForm)
+    
\ No newline at end of file
